# -*- coding: utf-8 -*-
"""
Linux Show Player

Cue player for live shows
"""

__author__ = 'Francesco Ceruti'
__email__ = 'ceppofrancy@gmail.com'
__url__ = 'https://github.com/FrancescoCeruti/linux-show-player'
__license__ = 'GPLv3'
__version__ = '0.5'
